
c#define AdjTemp
c#define TimedepAdjTemp


      SUBROUTINE surfch

C**** *surfch* - calculates gas exchange and chemistry at sea surface

C     e. maier-reimer      mpi hamburg      88/07/01.

C     purpose.
C     --------
C          *surfch* calculates gas exchange and chemistry at sea surface


C**   interface.
C     ----------
C          *call* *surfch*

C     method.
C     -------
C          solving system of two non-linear simultaneous equations
C     for [h2co3] and [h+] (expressed as ''bobsx''=SQRT(ak1*ak2)/[h+])
C     by newton-raphson method (e.g. scarborough, 1958), the surface
C     partial pressure ([h2co3]) and surface [h+] are determined
C     for the actual values of [alk] and [sum(co2)].
C     (bobsx has been used instead of [h+] for numerical reasons in
C     the bacastow version. in this version [h+] could be used instead
C     of bobsx.)
C          in a second iteration (also newton-raphson numerics) surface
C     pco2 and [h+] values at the actual time step which result
C     after exchange with the atmosphere are calculated.
C          values for surface [sum(co2)] and atmospheric pco2 are gained
C     by a simple piston velocity approach (stagnant film model)
C     using a given value for the exchange rate (''flura'').

C          the surface chemistry values have to be calculated at the
C     new time step for the ocean because of the fairly short time const
C     for gas exchange surface layer <-> atmosphere (several months,
C     cf. broecker and peng (1982):
C     time constant for equilibrium air/water for co2 about tenfold
C     as compared to o2 gas, i.e. ca. 10 months, p.150). if the surface
C     carbon system values would be calculated at the old time level
C     the system gets unstable.

C     externals.
C     ----------
C          none.

C     reference.
C     ----------

C          broecker, w.s., and t.-h. peng (1982)
C          tracers in the sea.
C          eldigio press, lamont-doherty geological observatory,
C          palisades, n.y., 690 pp..

C          mook, w.g. (1986)
C          13c in atmospheric co2.
C          netherlands journal of sea research, 20(2/3): 211-223.

C          scarborough, j. (1958) numerical mathematical analysis.
C          oxford university press, london, 4th ed., 576 pp..

C*     variable           type    purpose.
C      --------           ----    --------

C      *zeroab*           REAL    273.15 (0 deg c in deg k), half prec.
C      *onr*              REAL    1., half precision
C      *twr*              REAL    2., half precision
C      *fourr*            REAL    4., half precision
C      *evfr00*           REAL    coefficient of temperature dependent
C                                 isotopic carbon fractionation factor
C                                 for evaporation (mook, 1986)
C      *evfr01*           REAL    coefficient of temperature dependent
C                                 isotopic carbon fractionation factor
C                                 for evaporation (mook, 1986)
C      *fx*               REAL    local derivative of [alk]([h2co3],[h+]
C                                 with respect to [h+]
C      *fy*               REAL    local derivative of [alk]([h2co3],[h+]
C                                 with respect to [h2co3]
C      *gx*               REAL    local derivative of inorganic c as fun
C                                 of [h2co3] and [h+] with respect to [h
C      *gy*               REAL    local derivative of inorganic c as fun
C                                 of [h2co3] and [h+] with respect to [h
C      *deti*             REAL    determinant with local derivatives for
C                                 newton-raphson iteration
C      *ff1*              REAL    function [alk] given minus [alk] in te
C                                 of [h2co3] and [h+]
C      *cc1               REAL    inorganic carbon [mole/l] as function 
C                                 [h2co3] and [h+], dummy variable
C      *gg1               REAL    function inorg. c given minus inorg. c
C                                 pressed in terms of [h2co3] and [h+]
C      *s*                REAL    surface [h2co3], dummy variable
C      *testt*            REAL    check for convergence of newton-raphso
C                                 iteration
C      *xn*               REAL    surface [h+] (expressed as
C                                 SQRT(ak1*ak2)/[h+])
C                                 at END of one iteration step
C      *sss*              REAL    surface [h2co3] at END of one iteratio
C                                 step
C      *cto*              REAL    actual [sum(c-12)o2], dummy variable
C      *ct1*              REAL    actual [sum(c-12)o2], dummy variable
C      *alka*             REAL    actual [alk] [eqv/l], dummy variable
C      *dccdc*            REAL    carbonate alkalinity expressed in term
C                                 of [h2co3] and [h+]
C      *akb*              REAL    1. dissoc. constant of boric acid
C      *ddx*              REAL    resulting correction for [h+] at END
C                                 of one iteration step
C      *ddct*             REAL    resulting correction for [h2co3] at en
C                                 of one iteration step
C      *reduk*            REAL    damping factor of corrections in
C                                 newton-raphson algorithm to ensure
C                                 convergence
C      *h2co3(ie,je)*     REAL    surface [h2co3] [mole/l]
C      *bobsx(ie,je)*     REAL    [h+] expressed as SQRT(ak1*ak2)/[h+]
C                                 ak1 = 1. dissoc. const. of carb. acid
C                                 ak2 = 2. dissoc. const. of carb. acid
C      *deltax(ie,je)*    REAL    absolute value of correction for [h+]
C      *deltas(ie,je)*    REAL    asbolute value of correction for [h2co
C      *test1*            REAL    factor for sum of corrections of [h2co
C                                 to specify, when iteration can be STOP
C      *test2*            REAL    factor for sum of corrections of [h+]
C                                 to specify, when iteration can be STOP
C      *twr*              REAL    2., half precisionion
C      *fourr*            REAL    4., half precisionion
C      *year*             REAL    number of seconds in 1 year
C      *flura*            REAL    exchange rate of co2 through surface l
C                                 /atmosphere interface [year**-1]
C      *gamma*            REAL    fraction of [co2] passing through the
C                                 surface layer/atmsophere interface in 
C                                 time step
C      *krorr*         INTEGER    counts iterations in newton-raphson al
C                                 rithm
C      *alpha*            REAL    ak0, solubility of co2 in seawater, du
C                                 variable
C      *tt3*              REAL    SQRT(ak1*ak2), dummy variable
C      *bt*               REAL    total borate [moles/l]
C                                 ([b(oh)3]+[b(oh)4-]), dummy variable
C      *akk*              REAL    SQRT(ak1/ak2), dummy variable
C      *tt2*              REAL    akw/chemc(i,j,3), dummy variable
C      *tt1*              REAL    SQRT(ak1*ak2)/akb, dummy variable
C      *ct1*              REAL    actual value of inorg. c, dummy variab
C      *x1*               REAL    [h+] expressed as SQRT(ak1*ak2)/[h+],
C                                 dummy variable
C      *sumds*            REAL    sum of absolute values from all surfac
C                                 grid points for corrections
C                                 of [h2co3] at END of an iteration step
C      *sumdx*            REAL    sum of absolute values from all surfac
C                                 grid points for corrections
C                                 of [h+] at END of an iteration step
C      *pco2*             REAL    pco2 of surface water [ppm], dummy var
C      *pa*               REAL    atmospheric pco2 [ppm], dummy variable
C      *fugac*            REAL    enhancement factor for gas exchange fl
C      *fld*              REAL    [co2] flux atmosphere -> ocean
C                                 in [ppm/time step]
C      *flu*              REAL    [co2] flux ocean -> atmospher
C                                 in [ppm/time step]
C      *relw13*           REAL    ratio [sum((13c)o2)]/[sum((12c)o2)]
C                                 in surface layer
C      *rela13*           REAL    ratio [sum((13c)o2)]/[sum((12c)o2)]
C                                 in the atmosphere
C      *relw14*           REAL    ratio [sum((14c)o2)]/[sum((12c)o2)]
C                                 in surface layer
C      *rela14*           REAL    ratio [sum((14c)o2)]/[sum((12c)o2)]
C                                 in the atmosphere

C     -----------------------------------------------------------------
      INCLUDE 'parameter.include'
      INCLUDE 'common.horvel'
      INCLUDE 'common.wathic'
      INCLUDE 'common.chemis'
      INCLUDE 'common.biocon'
      INCLUDE 'common.topogr'
      INCLUDE 'common.production'
      INCLUDE 'common.thermo'
      INCLUDE 'common.ctrac'
      INCLUDE 'common.divers'
      INCLUDE 'common.caco3c'
      INCLUDE 'equivalence.include'
      REAL fx,fy,gx,gy,deti,ff1,cc1,gg1,s,x1,tt1,tt2,tt3,akk,bt,
     .  onr,twr ,xn,sss,cto,ct1,alka,dccdc,akb ,ddx,ddct,
     .  fourr,reduk,gamma
      REAL h2co3(ie,je),bobsx(ie,je)
      DIMENSION deltax(ie,je),deltas(ie,je)

      real delta_t
      save delta_t
      data delta_t /0/
      character*12 instring

#ifdef IDontNeedAnyOfThis
      scalefac = 4.3    !  DT2x / ln(2)
      delta_t_offset = 0.
      call getarg(3,instring)
      if(instring .EQ. 'forcet' .OR. instring .EQ. 'forcect') then
         call getarg(2,instring)
         if(instring .EQ. 'diss') then
            slugt = 25000
         else
            slugt = 6000
         endif
         if(nyear .GT. slugt) then
            call getarg(4,instring)
            if(instring .EQ. '1') then
               delta_t_offset = 1.
            elseif(instring .EQ. '2') then
               delta_t_offset = 2.
            endif
         endif
      endif
c      call getarg(1,instring)
c      if(instring .EQ. 'adjtt') then
c         delta_t_target = log(atc12(1,31) / 278.75) * scalefac
c     $        + delta_t_offset
c         delta_t = delta_t + (delta_t_target-delta_t) / 1000
c         write(6,*) "Delta T = ", delta_t_target, delta_t
c      elseif(instring .EQ. 'adjt') then
c         delta_t = log(atc12(1,31) / 278.75) * scalefac
c     $        + delta_t_offset
c         write(6,*) "Delta T = ", delta_t, atc12(1,31)
c      else
c         delta_t = delta_t_offset
c      endif
#endif

C     -----------------------------------------------------------------
C*         1. set half precision constants
C             --- ---- --------- ---------

      half=0.5
      zeroab=273.15
      onr=1.
      twr=2.00
      fourr=4.00

C     -----------------------------------------------------------------
C*         2. set factors for sum of corrections, to specify, when
C             iteration can be STOPped
C             (test1 for [h2co3], test2 for [h+])
C             ------ --- -------- ----- --- ----- -- -------- ----

      test1=1.e13
      test2=5.e8

C     -----------------------------------------------------------------
C*         3. set coefficients for temperature dependent
C             isotopic carbon fractionation factors
C             (mook, 1986, eq. 7a)
C             ------ ----- --- ------------ ------------

      evfr00=1.02389
      evfr01=9.483

C     -----------------------------------------------------------------
C*         4. set number of seconds in one year
C             --- ------ -- ------- -- --- ----

      year=365.242*86400.

C     -----------------------------------------------------------------
C*         5. co2 exchange rate per year between ocean and atmosphere.
C             flura = lambda/mp
C               where
C               lambda = (15 moles/300 ppm) /
C                        (1 m**2 * 1 year)
C                      = (15 moles/300 ppm) /
C                        (1.e4  cm**2 * 1 year)
C               mp = 33.e-6 moles/(ppm cm**2)
C               thus
C          15                            moles cm**2 ppm
C   flura= ----------------------------- --------------------
C          300*1e4*(365.242*86400)*33e-6 ppm cm**2 year moles
C                  (year in seconds)

C          15*1e2                  1
C        = ---------------------- ----
C          (365.242*86400)*300*33 year

C        ------------------------------- --- ----- ---- -----


      flura=1700./(year*270.*33.)


C     -----------------------------------------------------------------
C*         6. exchange co2 flux through water/air interface
C             per time step
C             -------- --- ---- ------- --------- ---------

        gamma=flura*dt

C     -----------------------------------------------------------------
C*         7. surface chemistry (pco2 and [h+] -via bobsx- in
C             surface layer); the result of this calculation
C             is used for output of oceanic pco2 and ph-value
C             to the cyber 860 for plotting these PARAMETERs
C             -- --- ----- --- --- -------- ----- -----------

C*         7.1 store [surface layer [h2co3] and [h+] as bobsx=
C              SQRT(ak1*ak2)/[h+] as first guess for iteration
      DO 21 j=jl1,jle
      DO 21 i=1,ie
         h2co3(i,j)=chemv(i,j,1)
         bobsx(i,j)=chemv(i,j,2)
21    CONTINUE
C     WRITE(6,67725) (j,(chemv(1,j,l),l=1,5),j=jl1,jle)

C*         7.2 iteration loop (krorr counts iterations)
      DO 9999 krorr=1,5000
      DO 99 j=jl1,jle
      DO 99 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 99
C     tt=tem(i,j,1)

C*         7.3 solubility of co2 in sea water
      alpha=chemc(i,j,5)

#ifdef AdjTempThisIsWrongIsntIt
      call getarg(1, instring)
      if(instring .EQ. 'adjt') then
         delta_sol = exp(delta_t * 0.04)
         alpha = alpha / delta_sol
      endif
#endif

C*         7.4 SQRT(ak1*ak2) (ak1, ak2 dissoc. const. of carb. acid)
      tt3=chemc(i,j,3)

C*         7.5 total borate
      bt=chemc(i,j,6)

C*         7.6 SQRT(ak1/ak2) (ak1, ak2 dissoc. const. of carb. acid)
      akk=chemc(i,j,4)

C*         7.7 akw/SQRT(ak1*ak2) (ak1, ak2 = dissoc. const. of carb.
C              acid, akw = ionic product of water)
      tt2=chemc(i,j,2)

C*         7.8 SQRT(ak1*ak2)/akb (ak1, ak2 = dissoc. const. of carb.
C              acid, akb = dissoc. const. of boric acid)
      tt1=chemc(i,j,1)

C*         7.9 calculate 1. dissociation of boric acid
      akb=chemc(i,j,3)/chemc(i,j,1)

C*        7.10 begin of newton-raphson algorithm:
C              (newton-raphson method:
C               the values of [sum(co2)] and [alk] are given,
C               desired roots of [h2co3] and [h+] for that pair
C               are determined by solving numerically the system
C               of the two nonlinear equations
C        1) [alk]given      - [alk]([h2co3],[h+])      = 0 (=ff1)
C        2) [sum(co2)]given - [sum(co2)]([h2co3],[h+]) = 0 (=gg1)

C*        7.11 set dummy variable for [alk]given and
C              [sum(co2)] given
      ct1=sco212(i,j,1)
      cto=ct1
      alka=alkali(i,j,1)

C*        7.12 set dummy variable for [h+], and [co3--]
      x1=bobsx(i,j)
      s=h2co3(i,j)

C*        7.13 carbonate alkalinity [hco3-]+2[co3--] expressed
C              in terms of [h2co3] and [h+]
      dccdc=s*(akk*x1+x1**2*twr)

C*        7.14 actual value of function ff1=[alk]given-[alk]([h2co3],[h+
      ff1=alka-dccdc-tt2*x1+tt3/x1-bt/(onr+tt1/x1)

C*        7.15 calculate [sum(co2)]([h2co3], [h+])
      cc1=s*(onr+akk*x1+x1**2)

C*        7.16 actual value of function gg1=[sum(co2)]given
C                                          -[sum(co2)]([h2co3],[h+])
      gg1=cto-cc1

C*        7.17 control variable to check convergence
      testt=ff1**2+gg1**2

C*        7.18 local derivative of [alk]([h2co3],[h+]) with
C              respect to [h+]
      fx=-s*(akk+fourr*x1)-tt2-tt3/x1**2
     .    -bt*tt1/(x1+tt1)**2

C*        7.19 local derivative of [alk]([h2co3],[h+]) with
C              respect to [h2co3]
      fy=-(akk*x1+twr*x1**2)

C*        7.20 local derivative of [sum(co2)]([h2co3],[h+]) with
C              respect to [h+]
      gx=-s*(akk+twr*x1)

C*        7.21 local derivative of [sum(co2)]([h2co3],[h+]) with
C              respect to [h2co3]
      gy=-(onr+akk*x1 +x1**2)

C*        7.22 solve system of linear equations for corrections
C              of [h2co3] and [h+] by determinants (''cramersche
C              regel'') and add corrections to [h2co3] and
C              [h+] from the preceeding step (reduk: damping
C              to ensure convergence)
      deti=onr/(fx*gy-fy*gx)
      ddx=deti*(gy*ff1-fy*gg1)
      ddct=deti*(fx*gg1-gx*ff1)
      reduk=onr/(onr+twr*((ddx/x1)**2+(ddct/s)**2))
C     IF(i.EQ.1.AND.MOD(j,7).EQ.0) WRITE(6,69916) j,ddx,x1,ddct,s
C    1   , reduk,cto,alka
      xn=x1-ddx *reduk
      sss=s-ddct *reduk

c      if(i .EQ. 1 .AND. j .EQ. 29) then
c         write(6,"(a7,i4,5g12.4)") "iter", krorr,
c     $        sss,ddct,reduk
c      endif

      bobsx(i,j)=xn
      h2co3(i,j)=sss

C*        7.23 check if corrections for [h2co3] and [h+] small enough
C              to stop iteration
      deltax(i,j)= abs(ddx)
      deltas(i,j)= abs(ddct)

C     IF(i.EQ.1) WRITE(6,691)s,x1,tt,ddx,ddct,alpha

99    CONTINUE

      sumdx=zero
      sumds=zero
      DO 95 j=jl1,jle
      DO 95 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 95
      sumds=sumds+deltas(i,j)
      sumdx=sumdx+deltax(i,j)
95    CONTINUE
C     WRITE(6,6971) krorr,sumdx,sumds
      IF(test1*sumds+test2*sumdx.LT.twr) GO TO 9998
9999  CONTINUE
9998  CONTINUE

c      do i=1,ie
c         do j=1,je
c            if(deltax(i,j) .GT. 1.e-6) then
c               write(6,*) "deltas", i,j,deltas(i,j)
c            endif
c         enddo
c      enddo

c      if(i .EQ. 6 .AND. j .EQ. 9) then
c      write(6,"(a7,i3,5g12.4)") "surfch",krorr,
c     $     xn,sss,reduk,bobsx(i,j),h2co3(i,j)
c      endif



C*        7.24 storage of actual values for surface [h2co3] and [h+]
C              (as bobsx=SQRT(ak1*ak2)/[h+]) and pco2 [ppm] on the
C              chemv-array
      DO 94 j=jl1,jle
      DO 94 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 94
      chemv(i,j,1)=h2co3(i,j)
      chemv(i,j,2)=bobsx(i,j)
      chemv(i,j,3)=chemv(i,j,1)/chemc(i,j,5)
      h2co3(i,j)=chemv(i,j,5)
      bobsx(i,j)=chemv(i,j,6)
94    CONTINUE

C     -----------------------------------------------------------------
C*         8. calculation of [h2co3] and [h+] (bobsx=SQRT(ak1*ak2)/[h+])
C             that result after exchange with the atmosphere.

C             a modified version of the method as for determining the
C             actual values of [h2co3] and [h+] is used.
C             the given value for [sum(co2)] is replaced by the sum
C             of the actual values of [sum(co2)] plus atmospheric [co2].
C             the corresponding PARAMETER in terms of [h2co3] and [h+]
C             contains also the sum [sum(co2)]([h2co3],[h+]) but
C             furthermore the corresponding co2 flux (''s*gamma'')
C             for compensation of the addition of atmospheric [co2]
C             to the surface layer [sum(co2)] value.
C             no corresponding modification for the alkalinity terms
C             is needed since the effect of gas exchange on [alk] is
C             negligible small.
C             ---------- -------- ------ -- --- -------- -- ----- --

C*         8.1 store [surface layer [h2co3] and [h+] as bobsx=
C              SQRT(ak1*ak2)/[h+] as first guess for iteration
      DO 941 j=jl1,jle
      DO 941 i=1,ie
      h2co3(i,j)=chemv(i,j,5)
      bobsx(i,j)=chemv(i,j,6)
941   CONTINUE

C*         8.2 iteration loop (krorr counts iterations)   no shit.
      DO 7777 krorr=1,30
C     WRITE(6,6891) krorr
      DO 78 j=jl1,jle
      DO 78 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 78

C     tt=tem(i,j,1)

C*         8.3 solubility of co2 in sea water
      alpha=chemc(i,j,5)
#ifdef AdjTempThisIsWrongToo
      delta_sol = exp(delta_t * 0.04)
      alpha = alpha / delta_sol
#endif

C*         8.4 SQRT(ak1*ak2) (ak1, ak2 dissoc. const. of carb. acid)
      tt3=chemc(i,j,3)

C*         8.5 total borate
      bt=chemc(i,j,6)

C*         8.6 SQRT(ak1/ak2) (ak1, ak2 dissoc. const. of carb. acid)
      akk=chemc(i,j,4)

C*         8.7 akw/SQRT(ak1*ak2) (ak1, ak2 = dissoc. const. of carb.
C              acid, akw = ionic product of water)
      tt2=chemc(i,j,2)

C*         8.8 SQRT(ak1*ak2)/akb (ak1, ak2 = dissoc. const. of carb.
C              acid, akb = dissoc. const. of boric acid)
      tt1=chemc(i,j,1)

C*         8.9 calculate 1. dissociation of boric acid
      akb=chemc(i,j,3)/chemc(i,j,1)

C*        8.10 exchange co2 flux through water/air interface
      gamma=flura*dt*rwat(i,j)

C*        8.11 begin of newton-raphson algorithm:
C              (newton-raphson method:
C               the values of [sum(co2)](+[co2](atm))
C               [sum(co2)] and [alk] are given,
C               desired roots of [h2co3] and [h+] for that pair
C               are determined by solving numerically the system
C               of the two nonlinear equations
C  1) [alk]given      -     [alk]([h2co3],[h+])          = 0 (=ff1)
C  2) ([sum(co2)] + atm.[co2])given) +
C     ([sum(co2)]([h2co3],[h+]) + atm.[co2] + comp.flux) = 0 (=gg1)

C   short form of eq. 2:
C                   (sum c)given - (sum c)([h2co3],[h+]) = 0 (=gg1)


C*        8.12 set dummy variable for [alk]given and
C              (sum c)given
      ct1=sco212(i,j,1)
      cto=ct1 +volrat*atc(i,j,1)
      alka=alkali(i,j,1)

C*        8.13 set dummy variable for [h+], and [co3--]
      x1=bobsx(i,j)
      s=h2co3(i,j)

C     IF(nyear.EQ.2) then
C     IF(i.EQ.1)WRITE(6,67723) alka,s,akk,x1,tt1,tt2,bt,akb
C     ENDIF

C*        8.14 carbonate alkalinity [hco3-]+2[co3--] expressed
C              in terms of [h2co3] and [h+]
      dccdc=s*(akk*x1+x1**2*twr)

C*        8.15 actual value of function ff1=
C                               =[alk]given-[alk]([h2co3],[h+])
      ff1=alka-dccdc-tt2*x1+tt3/x1-bt/(onr+tt1/x1)

C*        8.16 calculate (sum c)([h2co3], [h+])
C     cc1=s*(onr+akk*x1+x1**2) +volrat*(atc(i,j,1)+s*gamma
C    1/alpha)/(onr+gamma)
C new in crun66l 89/09/20 begin
      sol=chemv(i,j,1)
      gamh=gamma*half
      cc1=s*(onr+akk*x1+x1**2)+volrat*(atc(i,j,1)*(onr-gamh)
     .    +(s+sol)*gamh/alpha)/(onr+gamh)
C new in crun66l 89/09/20 END

C*        8.17 actual value of function gg1=(sum c)given
C                                          -(sum c)([h2co3],[h+])
      gg1=cto-cc1

C*        8.18 control variable to check convergence
      testt=ff1**2+gg1**2

C*        8.19 local derivative of [alk]([h2co3],[h+]) with
C              respect to [h+]
      fx=-s*(akk+fourr*x1)-tt2-tt3/x1**2
     .  -bt*tt1/(x1+tt1)**2

C*        8.20 local derivative of [alk]([h2co3],[h+]) with
C              respect to [h2co3]
      fy=-(akk*x1+twr*x1**2)

C*        8.21 local derivative of (sum c)([h2co3],[h+]) with
C              respect to [h+]
      gx=-s*(akk+twr*x1)

C*        8.22 local derivative of (sum c)([h2co3],[h+]) with
C              respect to [h2co3]
C     gy=-(onr+akk*x1 +x1**2)-gamma*volrat/(alpha*(onr+gamma))
C new in crun66l begin
      gy=-(onr+akk*x1 +x1**2)-gamh *volrat/(alpha*(onr+gamh ))
C new in crun66l END

C*        8.23 solve system of linear equations for corrections
C              of [h2co3] and [h+] by determinants (''cramersche
C              regel'') and add corrections to [h2co3] and
C              [h+] from the preceeding step (reduk: damping
C              to ensure convergence)
      deti=onr/(fx*gy-fy*gx)
      ddx=deti*(gy*ff1-fy*gg1)
      ddct=deti*(fx*gg1-gx*ff1)
      reduk=onr/(onr+twr*((ddx/x1)**2+(ddct/s)**2))
C     IF(i.EQ.1.AND.MOD(j,7).EQ.0) WRITE(6,67716) j,ddx,x1,ddct,s,
C    1reduk,cto,alka
      xn=x1-ddx *reduk
      sss=s-ddct *reduk
      bobsx(i,j)=xn
      h2co3(i,j)=sss

C*        8.24 check if corrections for [h2co3] and [h+] small enough
C              to STOP iteration
      deltax(i,j)= abs(ddx)
      deltas(i,j)= abs(ddct)

C     IF(i.EQ.1) WRITE(6,671)s,x1,tt,ddx,ddct,alpha

78    CONTINUE
      sumds=0.
      sumdx=0.
      DO 75 j=jl1,jle
      DO 75 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 75
      sumds=sumds+deltas(i,j)
      sumdx=sumdx+deltax(i,j)
75    CONTINUE
C     WRITE(6,6971) krorr,sumdx,sumds
      IF(test1*sumds+test2*sumdx.LT.twr) GO TO 7778
7777  CONTINUE
7778  CONTINUE

C*        8.25 storage of values for surface [h2co3] and [h+]
C              (as bobsx=SQRT(ak1*ak2)/[h+]) and ph-value that
C              are valid after gas exchange on the chemv-array
      DO 74 j=jl1,jle
      DO 74 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 74
      chemv(i,j,5)=h2co3(i,j)
      chemv(i,j,6)=bobsx(i,j)
      chemv(i,j,7)=chemv(i,j,5)/chemc(i,j,5)
      deltax(i,j)=chemc(i,j,3)/chemv(i,j,2)
      chemv(i,j,4)=-alog10(deltax(i,j))
74    CONTINUE
C     WRITE(6,68227) (((chemv(i,j,k),k=1,5),j=5,70,5),i=1,37,36)

C     -----------------------------------------------------------------
C*         9. gas exchange ocean surface layer <-> atmosphere

C             partial pressure difference between ocean and atmosphere
C             is calculated at the actual time level because
C             of the rather short time constant for exchange between
C             mixed layer and atmosphere (several months).

C        p(atm,new) = p(atm,old) + gamma*(p(oc,new)-p(atm,new))
C                     p(atm,old) + flura*(p(oc,new)-p(atm,new))*dt
C                                              ===        ===
C        ---------- - ---------- - -------------------------------------



      DO 199 j=jl1,jle
      DO 199 i=1,ie
         IF(rwat(i,j) .EQ. 0.) GO TO 199
         gamma=flura*dt*rwat(i,j)
         alpha=chemc(i,j,5)

#ifdef AdjTempThisIsWrongToo
      delta_sol = exp(delta_t * 0.04)
      alpha = alpha / delta_sol
#endif

         pco2=half*(chemv(i,j,5)+chemv(i,j,1))/alpha
         gamh=half*gamma
         pa=(atc(i,j,1)*(onr-gamh)+gamma*pco2)/(onr+gamh)
         fugac=1.
         fld=flura*pa*dt *rwat(i,j)*fugac
         flu=flura*pco2*dt*rwat(i,j)*fugac
         IF(rwat(i,j).NE.0.)relw13=sco213(i,j,1)/sco212(i,j,1)
         rela13=atc13(i,j)/atc12(i,j)
         fra13=evfr00-evfr01/(tem(i,j,1)+zeroab)
         fra14=fra13**2

c

         atc13(i,j)=atc13(i,j)-fld*rela13+flu*relw13*fra13
         atc12(i,j)=atc12(i,j)-fld+flu


         sco213(i,j,1)=sco213(i,j,1)+volrat*(fld*rela13-
     .        flu*relw13*fra13)

         IF(rwat(i,j).NE.0.)relw14=sco214(i,j,1)/sco212(i,j,1)

         rela14=(atc14(i,j)/atc12(i,j))
         atc14(i,j)=atc14(i,j)-fld*rela14+flu*relw14*fra14
         sco214(i,j,1)=sco214(i,j,1)+volrat*(fld*rela14  -
     .        flu*relw14*fra14)
         sco212(i,j,1)=sco212(i,j,1)+volrat*(fld-flu)
199   CONTINUE

      RETURN
6771  FORMAT(' ',i5,2e12.5)
6891  FORMAT(' iterat ',i5)
6971  FORMAT('  konver',i5,2e12.5)
62281 FORMAT(8e10.2)
62235 FORMAT(' krorr',i5)
67716 FORMAT(' ',i5,7e12.5)
67723 FORMAT(' ',8e10.3)
67725 FORMAT(i6,5e11.3)
68227 FORMAT(' ',5e12.5)
68623 FORMAT(' ',i5,7e12.5)
69118 FORMAT(' ',i3,2f6.1,4f6.4,2f6.1,2e10.3,f5.2)
69916 FORMAT(' ',i5,7e12.5)
      END
